#include "ammo.h"				// Required for ATI Remote Wonder plug-in SDK API

#define WM_REMOTEMSG (WM_APP+1)


// The number of mapped programmable functions provided by the plug-in
#define kNumFunctions				(6)

// List of strings for the programmable keys (A-F). Each index in this list
// is mapped to a function in the defintions above.
static char *functions[kNumFunctions] = 
{	
	// To create groups in the tree control use the "|" symbol to seperate
	// the branch from the root. For Winamp, we have three types of groups.
	// The general options are put into the root while settings that are
	// toggled and equalizer settings each have a seperate branch.

	// All of the general options are at the root level.
	"Button A",
	"Button B",
	"Button C",
	"Button D",	
	"Button E",
	"Button F",
};

/*
 * Function:	WhatKeysDoYouWant
 *
 * Purpose:		Determines what groups of keys should be re-directed to
 *				the plug-in.
 *
 * Inputs:		None
 *
 * Returns:		A bit mask which indicates the groups of keys that should
 *				be re-directed to this plug-in.
 *
 * When called: This function is called once when the plug-in is first loaded,
 *				and again after calling Configure().
 */
DWORD WhatKeysDoYouWant (void)
{
	HKEY hkey = NULL;
	DWORD dwOverrideMouse = 0;
	DWORD dwReturn;

	if( ERROR_SUCCESS == RegOpenKeyEx(HKEY_CURRENT_USER, "Software\\ATI Remote Wonder Plugin For Girder", 0, KEY_READ, &hkey ) )
	{
		DWORD dwType = 0;
		DWORD dwSize = sizeof(DWORD);
		RegQueryValueEx( hkey, "Override Mouse", 0, &dwType, (BYTE*) &dwOverrideMouse, &dwSize );
		RegCloseKey( hkey );
	}

	// For girder we want all commands.  Use CUSTOM_MAPPED to get mapped commands 
	dwReturn = (CUSTOM_MAPPED|CHANNEL_GROUP|VOLUME_GROUP|NUMBER_GROUP|CURSOR_GROUP|
				PLAY_GROUP|MENU|SETUP|ENTER|RECORD|STOPWATCH|RESIZE|WEB_LAUNCH);

	if( dwOverrideMouse == 1 )
		dwReturn |= MOUSE_GROUP;

	return dwReturn;
}


/*
 * Function:	EnumerateProgrammableFunction
 *
 * Purpose:		Enumerate the descriptions of the programamble
 *				functions provided by this plug-in.
 *
 * Inputs:		wIndex - index of requested description
 *
 * Returns:		Pointer to the programmable function description
 *				or NULL if wIndex is greater than the number of
 *				programmable functions provided by this plug-in.
 *
 * When called:	This function is called after any call to WhatKeysDoYouWant()
 *				that returns a request for custom mapped keys (CUSTOM_MAPPED).
 */
char *EnumerateProgrammableFunction (WORD wIndex)
{	
	if (wIndex >= kNumFunctions) 
		return NULL;

	// Return a pointer to the requested description (see table above)
	return functions[wIndex];
}


/*
 * Function:	Configure
 *
 * Purpose:		Displays any configuration/setup page required by
 *				this plug-in. 
 *
 * Inputs:		hWnd - handle to the parent window. The parent window
 *				is used to ensure correct modal behaviour when creating 
 *				dialog and message boxes in this plug-in.
 *
 * Returns:		None
 *
 * When called: This function is called when the "Configure" button in the
 *				ATI Remote Wonder software is pressed.
 */
void Configure (HANDLE hWnd)
{
	DWORD dwOverrideMouse = 0;
	HKEY hkey = NULL;

	// Display an 'About' message box
	int nResult = MessageBox(hWnd, "Do you want to override mouse movement?", "Girder Plug-In For ATI Remote Wonder Software v1.1", MB_YESNO);	
	if( nResult == IDYES )
		dwOverrideMouse = 1;
	else 
		dwOverrideMouse = 0;

	if( ERROR_SUCCESS == RegCreateKeyEx(HKEY_CURRENT_USER, "Software\\ATI Remote Wonder Plugin For Girder", 0, NULL,
						REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hkey, NULL ) )
	{
		RegSetValueEx( hkey, "Override Mouse", 0, REG_SZ, (BYTE*) &dwOverrideMouse, sizeof(DWORD) );
		RegCloseKey( hkey );
	}
}


/*
 * Function:	AreYouInFocus
 *
 * Purpose:		Determines if this plug-in is currently in-focus.
 *
 * Inputs:		None
 *
 * Returns:		TRUE if the plug-in is currently in focus, FALSE
 *				otherwise.
 *
 * When called:	This function is called when a key event occurs that
 *				the plug-in has requested.
 */
int AreYouInFocus (void)
{
	// Get the girder plugin's window.
	HWND hGriderPlugin = FindWindow("ATI Remote Wonder Plugin For Grider Class", NULL);

	// If the main window is not found, then girder's ATI Remote Wonder software hasn't started
	if (hGriderPlugin == NULL)
		return FALSE;

	return TRUE;
}


/*
 * Function:	HandleKey
 *
 * Purpose:		Handles remote control keys.
 *
 * Inputs:		bCustom   - TRUE if wKeyEvent has been mapped to a custom
 *						    programamble option.
 *				wKeyEvent - Remote control key code for button that was
 *							pressed.
 *				wState    - RMCTRL_KEY_ON, when button is first pressed
 *						    RMCTRL_KEY_REPEAT, when button is held down
 *						    RMCTRL_KEY_OFF, when button is released
 *
 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.
 *
 * When called: This function is called whenever a key event occurs that
 *				the plug-in has requested when the plug-in is in focus.
 */
BOOL HandleKey (BOOL bCustom, WORD wKeyEvent, WORD wState)
{
	HWND hGriderPlugin;

	// Get the girder plugin's window.
	hGriderPlugin = FindWindow("ATI Remote Wonder Plugin For Grider Class", NULL);

	// If the main window is not found, then girder's ATI Remote Wonder software hasn't started
	if (hGriderPlugin == NULL)
		return FALSE;

	if( bCustom )
	{
		switch(wKeyEvent)
		{
			case 0: wKeyEvent = RMCTRL_A; break;
			case 1: wKeyEvent = RMCTRL_B; break;
			case 2: wKeyEvent = RMCTRL_C; break;
			case 3: wKeyEvent = RMCTRL_D; break;
			case 4: wKeyEvent = RMCTRL_E; break;
			case 5: wKeyEvent = RMCTRL_F; break;
		}
	}
	
	// Send the mesage with what's happened
	SendMessage( hGriderPlugin, WM_REMOTEMSG, wKeyEvent, wState );

	return TRUE;
}

